﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;

namespace Phoenix.Xna.Framework.Reflection
{
    /// <summary>
    /// Extends the functionality of System.Reflection.Assembly instances.
    /// </summary>
    public static class AssemblyExtensions
    {
        /// <summary>
        /// Get all attributes marked on this assembly of a specific type.
        /// </summary>
        /// <typeparam name="A">The type of attributes to find.</typeparam>
        /// <param name="assembly">The extended object.</param>
        /// <param name="inherit">This argument is ignored for types of System.Reflection.Assembly.</param>
        /// <returns>The first attribute of the specified type.</returns>
        public static A GetFirstAttributeOfType<A>(this Assembly assembly, bool inherit)
            where A : Attribute
        {
            return assembly.GetAllAttributeOfType<A>(inherit).FirstOrDefault();
        }

        /// <summary>
        /// Get all attributes marked on this assembly of a specific type.
        /// </summary>
        /// <typeparam name="A">The type of attributes to find.</typeparam>
        /// <param name="assembly">The extended object.</param>
        /// <param name="inherit">This argument is ignored for types of System.Reflection.Assembly.</param>
        /// <returns>A list of attributes of the specified type.</returns>
        public static List<A> GetAllAttributeOfType<A>(this Assembly assembly, bool inherit)
           where A : Attribute
        {
            return assembly.GetCustomAttributes(typeof(A), inherit)
                .OfType<A>()
                .ToList();
        }
    }
}
