﻿using System;
using System.Reflection;
using System.Xml.Linq;

namespace Phoenix.Xna.Framework.Serialization.Markup.Properties
{
    /// <summary>
    /// Base class for all IMarkupProperty types.
    /// </summary>
    /// <typeparam name="T">The type of the internal IMarkupObject.</typeparam>
    internal abstract class MarkupPropertyBase<T> : IMarkupProperty
    {
        protected T _value;   // The value of the markup object.

        #region IMarkupElement Members

        /// <summary>
        /// Gets or sets the SerializationService that provides context to references, etc.
        /// </summary>
        public Serializer Serializer { get; set; }

        /// <summary>
        /// Serialize the element to XML.
        /// </summary>
        /// <returns>The element markup serialzed as an XElement.</returns>
        public abstract XElement Serialize();

        /// <summary>
        /// Deserialize the element from an XElement.
        /// </summary>
        /// <param name="element">The XElement to deserialize this element from.</param>
        public abstract void Deserialize(XElement element);

        #endregion

        #region IMarupProperty Members

        /// <summary>
        /// Gets or sets the name of the property.
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// Gets or sets the System.Type of the defined object.
        /// </summary>
        public Type ObjectType { get; set; }

        /// <summary>
        /// Gets a value indicating this property is not default, and should be serialized.
        /// </summary>
        public bool ShouldPersist { get; protected set; }

        /// <summary>
        /// Extract the markup information from the PropertyInfo for this property.
        /// </summary>
        /// <param name="field">The PropertyInfo for this field.</param>
        /// <param name="value">The value of the property.</param>
        public abstract void ExtractMarkup(PropertyInfo property, object value);

        /// <summary>
        /// Sets the value of MarkupProperty to the instance's property value.
        /// </summary>
        /// <param name="instance">The instance to set the property on.</param>
        public abstract void SetValue(object instance);

        #endregion
    }
}
